﻿using System;
using System.Collections.Concurrent;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using DTMF_IVR.Model;
using DTMF_IVR.Model.InputInterpreter;
using DTMF_IVR.Model.IVRMenuElements;
using DTMF_IVR.Windows;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.SDK;
using System.Collections.Generic;

namespace DTMF_IVR
{
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
        }

        private void Tsmi_HelpPage_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://www.voip-sip-sdk.com/");
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
        }

        private void Tsmi_About_Click(object sender, EventArgs e)
        {
            AboutBox aboutBox = new AboutBox();

            aboutBox.ShowDialog(this);
        }

        private IVREngine ivrEngine;


        private void FormMain_Load(object sender, EventArgs e)
        {
            Tssl_Ozeki.Text = string.Format("© Copyright {0}, Ozeki Ltd.", DateTime.Now.Year);

            ISoftPhone softPhone=null;

            /* Create a softphone...
             * */
            try
            {
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5700, 5750, 5700);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, @"Please read the DTMF IVR manual.");

                var sb = new StringBuilder();
                sb.AppendLine("Some error happened.");
                sb.AppendLine();
                sb.AppendLine("Exception:");
                sb.AppendLine(ex.Message);
                sb.AppendLine();
                if (ex.InnerException != null)
                {
                    sb.AppendLine("Inner Exception:");
                    sb.AppendLine(ex.InnerException.Message);
                    sb.AppendLine();
                }
                sb.AppendLine("StackTrace:");
                sb.AppendLine(ex.StackTrace);

                MessageBox.Show(sb.ToString());
            				
                return;
            }

            /* Set the SIP Account settings..
             * */
            SIPAccount acc = new SIPAccount(
                   registrationRequired: true,
                   displayName: "oz875",
                   userName: "oz875",
                   registerName: "oz875",
                   registerPassword: "oz875",
                   domainServerHost: "192.168.112.100",
                   domainServerPort: 5060
               );

            Dictionary<IPhoneCall, CallSoundEngine> soundEngines = new Dictionary<IPhoneCall, CallSoundEngine>();

            /* Create IVRMenu
             * */
            IIVRMenuElement rootMenu = CreateIVRMenu(soundEngines);

            /* Create IVREngine
             * */
            ivrEngine = new IVREngine(soundEngines, softPhone, rootMenu, acc);


            IVREngine.NewEngineEvent += IVREngine_NewReportEvent;

            /* Start IVREngine
             * */
            ivrEngine.Start();
        }

        private IIVRMenuElement CreateIVRMenu(Dictionary<IPhoneCall, CallSoundEngine> soundEngines)
        {
            VoiceMessageData voiceMessageData = new VoiceMessageData("email@example.com", "email@example.com", "subject", "body");

            /*Create IVR root menu
             */
            IIVRMenuElement rootMenu = new WavePlayerElement(soundEngines, "..\\..\\Resources\\demo.wav");

            /*Create IVR submenus
             */
            IIVRMenuElement dataBaseTextToSpechElement = new DataBaseTextToSpeechElement(soundEngines);
            IIVRMenuElement voiceMessageToEmailElement = new VoiceMessageToEmailElement(soundEngines, "mail.example.com", voiceMessageData);
            IIVRMenuElement softPhoneElement = new SoftPhoneElement(soundEngines);

            /*Adding submenus to the root menu
             */
            rootMenu.AddSubMenuElement(1, voiceMessageToEmailElement, new DTMFInputInterpreter());
            rootMenu.AddSubMenuElement(2, dataBaseTextToSpechElement, new DTMFInputInterpreter());
            rootMenu.AddSubMenuElement(3, softPhoneElement, new DTMFInputInterpreter());
            Lb_Report.Items.Add("-------------------------");
            Lb_Report.Items.Add("Sample IVR menu stucture:");
            Lb_Report.Items.Add("----------");
            Lb_Report.Items.Add("root menu (demo.wav playback)");
            Lb_Report.Items.Add("  1 - voice message to email menu");
            Lb_Report.Items.Add("      9 - exit");
            Lb_Report.Items.Add("  2 - databese textToSpeech menu");
            Lb_Report.Items.Add("      9 - exit");
            Lb_Report.Items.Add("  4 - softphone menu");
            Lb_Report.Items.Add("      9 - exit");
            Lb_Report.Items.Add("-------------------------");
            /*Create IVRMenu end element
             */
            IIVRMenuElement endMenuElement = new EndElement(soundEngines);

            /*Adding end menu to the submenus
             */
            rootMenu.AddSubMenuElement(9, endMenuElement, new DTMFInputInterpreter());
            dataBaseTextToSpechElement.AddSubMenuElement(9, endMenuElement, new DTMFInputInterpreter());
            voiceMessageToEmailElement.AddSubMenuElement(9, endMenuElement, new DTMFInputInterpreter());
            softPhoneElement.AddSubMenuElement(9, endMenuElement, new DTMFInputInterpreter());
            return rootMenu;
        }

        void IVREngine_NewReportEvent(object sender, EngineInformation e)
        {
            BeginInvoke((Action)(() => Lb_Report.Items.Add(e.EngineElementName + " - " + e.CallId + " - " + e.ReportString)));
        }

        private void bt_Start_Click(object sender, EventArgs e)
        {
            ivrEngine.Start();

            bt_Stop.Enabled = true;
            bt_Start.Enabled = false;
        }

        private void bt_Stop_Click(object sender, EventArgs e)
        {
            ivrEngine.Stop();

            bt_Stop.Enabled = false;
            bt_Start.Enabled = true;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
    }
}
