﻿using System.Collections.Concurrent;
using System.Linq;
using System.Text;
using DTMF_IVR.DataBaseModel;
using DTMF_IVR.Model.InputInterpreter;
using Ozeki.Media.MediaHandlers;
using Ozeki.VoIP;
using System.Collections.Generic;

namespace DTMF_IVR.Model.IVRMenuElements
{

    /// <summary>
    /// TODO
    /// </summary>
    internal class DataBaseTextToSpeechElement : BaseIVRMenuElement
    {
        private readonly DatabaseExampleEntities1 exampleEntities1;
        private TextToSpeech TextToSpeech;

        public DataBaseTextToSpeechElement(
            Dictionary<IPhoneCall, CallSoundEngine> soundEngines, 
            IInputInterpreter inputInterpreter = null)
            : base(soundEngines, inputInterpreter)
        {
            exampleEntities1 = new DatabaseExampleEntities1();

            TextToSpeech = new TextToSpeech();
        }

        /// <summary>
        /// DeActivate of the engine.
        /// </summary>
        public override void EngineStop()
        {
            TextToSpeech.StopStreaming();
            TextToSpeech.Clear();

            CallSoundEngine soundEngine = null;
            if (ItsSoundEngines.TryGetValue(PhoneCall, out soundEngine))
            {
                soundEngine.Connector.Disconnect(TextToSpeech, soundEngine.OutgoingAudioMixer);
            }

            PhoneCall.CallStateChanged -= PhoneCallCallStateChanged;
            exampleEntities1.Dispose();

            IVREngine.OnEngineEvent("DataBaseTextToSpech", PhoneCall.DialInfo.UserName, "Stop");
        }


        /// <summary>
        /// Activation of the engine.
        /// </summary>
        /// <param name="phoneCall"></param>
        public override void EngineExecute(IPhoneCall phoneCall)
        {
            PhoneCall = phoneCall;

            if (phoneCall == null || phoneCall.CallState != CallState.InCall)
                return;

            IVREngine.OnEngineEvent("DataBaseTextToSpech", phoneCall.DialInfo.UserName, "Start");
            phoneCall.CallStateChanged += PhoneCallCallStateChanged;

            CallSoundEngine soundEngine = null;
            if (!ItsSoundEngines.TryGetValue(phoneCall, out soundEngine))
                return;

            soundEngine.Connector.Connect(TextToSpeech, soundEngine.OutgoingAudioMixer);
            TextToSpeech.AddAndStartText(GetDataBaseInfo());

            ActivateInputInterpreter();
        }

        /// <summary>
        /// Handles the StateChanged event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhoneCallCallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            if (e.Item != CallState.InCall)
            {
                IPhoneCall call = (IPhoneCall)sender;
                EngineStop();
                IVREngine.OnEngineEvent("DataBaseTextToSpech", PhoneCall.DialInfo.UserName, "Hangup");
            }
        }

        /// <summary>
        /// Get database information
        /// </summary>
        /// <returns></returns>
        private string GetDataBaseInfo()
        {
            StringBuilder result = new StringBuilder();
            IQueryable<string> data = exampleEntities1.TableExamples.Select(e => e.Name);

            foreach (string row in data)
                result.Append(row + ", ");

            return result.ToString();
        }

        #region Cloning

        public override object Clone()
        {
            BaseIVRMenuElement element = new DataBaseTextToSpeechElement(ItsSoundEngines);
            BaseCloning(element);
            return element;
        }

        #endregion
    }
}